#pragma texture /code_random/index4.glsl


#define R .04
#define P_COUNT 17
#define DC_COUNT 16
const vec4 dc[DC_COUNT] = vec4[DC_COUNT](
    vec4(3,2,0.14999999202035977,0.1),
    vec4(4,3,0.099999989713187,0.1),
    vec4(5,4,0.050000011920928955,0.1),
    vec4(6,4,0.04999998940239111,0.1),
    vec4(7,3,0.09999999403953552,0.1),
    vec4(8,7,0.05000001521195773,0.1),
    vec4(9,7,0.05000001521195773,0.1),
    vec4(10,2,0.14999999202035977,0.1),
    vec4(11,10,0.09999999403953552,0.1),
    vec4(12,11,0.050000000310804184,0.1),
    vec4(13,11,0.050000000310804184,0.1),
    vec4(14,10,0.10000001552275858,0.1),
    vec4(15,14,0.049999974501229866,0.1),
    vec4(16,14,0.050000011920928955,0.1),
    vec4(0,1,0.000999998301267624,0.9),
    vec4(1,2,0.14900000393390656,0.9)
);

#define AC_COUNT 15
#define PC_COUNT 2

float sdCircle( vec2 p, float r )
{
    return length(p) - r;
}

float sdBox( in vec2 p, in vec2 b )
{
    vec2 d = abs(p)-b;
    return length(max(d,0.0)) + min(max(d.x,d.y),0.0);
}

float sdSegment( in vec2 p, in vec2 a, in vec2 b )
{
    vec2 pa = p-a, ba = b-a;
    float h = clamp( dot(pa,ba)/dot(ba,ba), 0.0, 1.0 );
    return length( pa - ba*h );
}


float hash11(float p)
{
    p = fract(p * .1031);
    p *= p + 33.33;
    p *= p + p;
    return fract(p);
}


#define P_COLOR 1.0
#define DC_COLOR 2.0
#define AC_COLOR 3.0
#define BOUNDS_COLOR 4.0

vec2 getInfo(int index) {
    vec4 info = texture(iChannel0, vec2((float(index)+.5)/iResolution.x, 0));
    info.xy = info.xy * 2. - 1.;
    
    return info.xy;
}

vec2 opU( vec2 d1, float d2, float material )
{
    return (d1.x<d2) ? d1 : vec2(d2, material);
}

vec2 map(vec2 p){
    vec2 d = vec2(99999.9, 0);
    for(int i = 0;i < P_COUNT;i ++) {
       d = opU(d, sdCircle(p - getInfo(i), R), P_COLOR);
    }

    for(int i = 0;i < DC_COUNT;i ++) {
      int a = int(dc[i][0]);
      int b = int(dc[i][1]);
      d = opU(d, sdSegment(p, getInfo(a).xy,  getInfo(b).xy), AC_COLOR);
    }
    
    d = opU(d, sdSegment(p, vec2(-1, -1), vec2(-1, 1)), BOUNDS_COLOR);
    d = opU(d, sdSegment(p, vec2(-1, 1), vec2(1, 1)), BOUNDS_COLOR);
    d = opU(d, sdSegment(p, vec2(1, 1), vec2(1, -1)), BOUNDS_COLOR);
    d = opU(d, sdSegment(p, vec2(1, -1), vec2(1, 1)), BOUNDS_COLOR);

    return d;
}

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
    vec2 uv = (2. * fragCoord - iResolution.xy) / iResolution.y;
    
    float w = length(fwidth(uv)) * 5.0;
    
    vec2 d = map(uv);
    float c = smoothstep(0.0, w, abs(d.x));
    vec3 col = vec3(1.0-c);
    if (d.y == P_COLOR) {
      col *= vec3(.3, .9, .6);
    } else if (d.y == DC_COLOR) {
      col *= vec3(.3, .6, .9);
    } else if (d.y == AC_COLOR) {
      col *= vec3(.6, .9, .3);
    } else if (d.y == BOUNDS_COLOR) {
      col *= vec3(.5, .5, .5);
    }

    col = mix(col, vec3(1, 1, .95), c/4.0);

    fragColor = vec4(col, 1);
}